<?php
/**
 * Created by PhpStorm.
 * User: matheussilva
 * Date: 17/09/14
 * Time: 17:53
 */

namespace Folha\Logic\ImportacaoLoteEvento;

use Folha\Entities\FpaGozoferias;
use Folha\Entities\GpeAltesitufunc;
use \Folha\Entities\GpeSitufunch;
use \Folha\Entities\GpeVinculoh;
use \Folha\Entities\GpeVinculom;
use \Geral\Entities\Estruturam;

class ExportarArquivo {

    private $nrorg;
    private $dtmescompetencia;
    private $nrestrututlegal;
    private $nrestrututgeren;
    private $nrestrututsind;
    private $vinculos = array();
    private $eventos = array();

    //CONSTANTES

    const CARACTER_ACENTUADO =     'ÀÁÂÃÄÅÆÇÈÉÊËÌÍÎÏÐÑÒÓÔÕÖØÙÚÛÜÝÞßàáâãäåæçèéêëìíîïðñòóôõöøùúûýýþÿ';
    const CARACTER_NAO_ACENTUADO = 'AAAAAAACEEEEIIIIDNOOOOOOUUUUYBSAAAAAAACEEEEIIIIDNOOOOOOUUUYYBY';
    const APENAS_CARACTERES_ALFANUMERICOS = '/[^a-zA-Z0-9\s]/';
    const CARACTER_BRANCO = ' ';
    const CARACTER_VAZIO = '';

    /**
     * Construtor da classe
     * @param \DateTime $dtmescompetencia
     * @param array $eventos
     * @param integer $nrorg
     * @param string $nrestrututlegal
     * @param string $nrestrututgeren
     * @param string $nrestrututsind
     *
     * @throws \Exception
     */
    function __construct($dtmescompetencia, $eventos, $nrorg, $nrestrututlegal = '', $nrestrututgeren = '', $nrestrututsind = '')
    {
        try {
            $this->eventos = $eventos;
            $this->dtmescompetencia = $dtmescompetencia;
            $this->nrestrututgeren = $nrestrututgeren;
            $this->nrestrututlegal = $nrestrututlegal;
            $this->nrestrututsind = $nrestrututsind;
            $this->nrorg = $nrorg;

            if (empty($nrestrututgeren) && empty($nrestrututlegal) && empty($nrestrututsind)) {
                throw new \Exception('Não foi selecionado nenhuma estrutura como filtro.', '11');
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Busca os vínculos que entrarão na planilha
     * @throws \Exception
     */
    private function getVinculos()
    {
        try {
            $where = array();
            if (!empty($this->nrestrututlegal)) {
                $where[] = 'NRESTRUTLEGAL = ' . $this->nrestrututlegal;
            }
            if (!empty($this->nrestrututgeren)) {
                $where[] = 'NRESTRUTGEREN = ' . $this->nrestrututgeren;
            }
            if (!empty($this->nrestrututsind)) {
                $where[] = 'NRESTRUTSIND = ' . $this->nrestrututsind;
            }

            $where = implode(' AND ', $where);

            //BUSCA O VÍNCULOH
            $objsVinculoh = $this->buscaVinculoh($where);

            $nrestrututgerenvinculo = 0;
            foreach ($objsVinculoh as $objVinculoh) {

                if ($nrestrututgerenvinculo != $objVinculoh->getNrestrutgeren()) {
                    $nrestrututgerenvinculo = $objVinculoh->getNrestrutgeren();
                    if ($this->vinculos[$nrestrututgerenvinculo]) {
                        $this->vinculos[$nrestrututgerenvinculo] = array();
                    }
                }

                //BUSCA O VÍNCULOM
                $objVinculom = $this->buscaVinculom($objVinculoh);

                if($this->buscaFerias($objVinculom)){
                    $nrsitufuncm = '100003';
                }else{
                    $nrsitufuncm = $this->buscaAlteSitufunc($objVinculom);
                }

                //BUSCA SITUFUNCM
                $objSitufunc = $this->buscaSitufuncm($nrsitufuncm);

                $this->vinculos[$nrestrututgerenvinculo][] = array('NRVINCULOM' => $objVinculom->getNrvinculom(),
                    'NMVINCULOM' => $objVinculom->getNmvinculom(),
                    'CDMATRICULA' => $objVinculom->getCdmatricula(),
                    'NMSITUFUNCH' => $objSitufunc->getNmsitufunch(),
                    'DTADMISSAOVINC' => $objVinculom->getDtadmissaovinc());

            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Busca o nome da estrutura através do nome
     * @param $nrestruturam
     * @param $nrorg
     * @return array|bool
     * @throws \Exception
     */
    private function getEstrutura($nrestruturam, $nrorg)
    {
        try {
            $params = array('P_NRESTRUTURAM' => $nrestruturam,
                'P_NRORG' => $nrorg);
            $where = "NRESTRUTURAM = :P_NRESTRUTURAM AND NRORG = :P_NRORG";
            $objeto = Estruturam::getObjectByWhere($where, $params);
            if ($objeto)
                return array('NRESTRUTURAM' => $objeto->getNrestruturam(),
                    'NMESTRUTURAM' => $objeto->getNmestruturam());
            else
                return false;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Recolhe os eventos selecionados pelo operador
     * @throws \Exception
     */
    private function getEvento()
    {
        try {
            $eventos = array();
            foreach ($this->eventos as $evento) {
                $eventos[] = array('NREVENTOM' => $evento['nreventom'],
                    'NMEVENTOM' => $evento['nmeventoh']);
            }
            $this->eventos = $eventos;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Gera o Arquivo de XLS
     * @param $vinculos
     * @param $nrestrututgeren
     * @throws \Exception
     */
    private function geraXls($vinculos, $nrestrututgeren)
    {
        try {
            $file = new \PHPExcel();
            $file->setActiveSheetIndex(0);

            $file->getActiveSheet()->setCellValue('A1', 'Estrutura Legal');
            $file->getActiveSheet()->setCellValue('B1', $this->nrestrututlegal['NRESTRUTURAM']);
            $file->getActiveSheet()->setCellValue('C1', $this->nrestrututlegal['NMESTRUTURAM']);

            $file->getActiveSheet()->setCellValue('A2', 'Estrutura Gerencial');
            $file->getActiveSheet()->setCellValue('B2', $this->nrestrututgeren['NRESTRUTURAM']);
            $file->getActiveSheet()->setCellValue('C2', $this->nrestrututgeren['NMESTRUTURAM']);

            $file->getActiveSheet()->setCellValue('A3', 'Competencia');
            $file->getActiveSheet()->setCellValue('B3', '##/##/####');

            $nrcol = 'A';
            $nrlin = 5;

            $colTitulos = array('Código do Vínculo',
                'Matrícula',
                'Vínculo',
                'Situação Funcional',
                'Data de Admissão');

            foreach ($colTitulos as $colTitulo) {
                $file->getActiveSheet()->setCellValue($nrcol++ . $nrlin, $colTitulo);
            }

//        $file->getActiveSheet()->setCellValue($nrcol++.$nrlin, 'Matrícula');
//        $file->getActiveSheet()->setCellValue($nrcol++.$nrlin, 'Vínculo');

            //INSERE OS VÍNCULOS
            $qtd = count($vinculos);
            for ($c = 0, $li = $nrlin + 1; $c < $qtd; $c++, $li++) {
                $file->getActiveSheet()->setCellValue('A' . $li, $vinculos[$c]['NRVINCULOM']);
                $file->getActiveSheet()->getCell('B' . $li)->setDataType();
                $file->getActiveSheet()->setCellValue('B' . $li, $vinculos[$c]['CDMATRICULA']);
                $file->getActiveSheet()->setCellValue('C' . $li, $vinculos[$c]['NMVINCULOM']);
                $file->getActiveSheet()->setCellValue('D' . $li, $vinculos[$c]['NMSITUFUNCH']);
                $file->getActiveSheet()->setCellValue('E' . $li, $vinculos[$c]['DTADMISSAOVINC']->format('d/m/Y'));

            }

            //INSERE OS EVENTOS

            for ($c = 0; $c < count($this->eventos); $c++, $nrcol++) {
                $file->getActiveSheet()->setCellValue($nrcol . '5', $this->eventos[$c]['NREVENTOM'] . ' - ' . $this->eventos[$c]['NMEVENTOM']);
            }

            $file->getActiveSheet()->setCellValue(++$nrcol . '5', 'Observação');

            //ARQUIVO

            $nomeArquivo = "Formulario_";
            $nomeArquivo .= self::_removeCaracteresEspeciais($this->nrestrututgeren['NMESTRUTURAM'])."_";
            $nomeArquivo .= $this->nrestrututgeren['NRESTRUTURAM'];
            $nomeArquivo .= '.xls';

            $expDir = APP_PATH . '/uploads/' . $this->nrorg . '/Exportacao/';
            if (!file_exists($expDir)) {
                mkdir($expDir, 0777, true);
            }
            $expDir = realpath($expDir);
            $write = \PHPExcel_IOFactory::createWriter($file, 'Excel5');
            $write->save($expDir . DIRECTORY_SEPARATOR . $nomeArquivo);

            $file = new \TkFile($expDir . DIRECTORY_SEPARATOR . $nomeArquivo);
            $file->download();
        } catch (\Exception $e) {
            throw $e;
        }

    }

    /**
     * Prepara e executa a exportação
     * @throws \Exception
     */
    public function gerar()
    {
        try {
            $this->getVinculos();
            $this->getEvento();
            $this->nrestrututlegal = $this->getEstrutura($this->nrestrututlegal, $this->nrorg);
            $this->nrestrututsind = $this->getEstrutura($this->nrestrututsind, $this->nrorg);
            foreach ($this->vinculos as $nrestrututlegal => $vinculos) {
                $this->nrestrututgeren = $this->getEstrutura($nrestrututlegal, $this->nrorg);
                $this->geraXls($vinculos, $nrestrututlegal);
            }
        } catch (\Exception $e) {
            throw $e;
        }


    }

    /**
     * Busca o vínculo a ser gravado na planília
     * @param GpeVinculoh $objVinculoh
     * @return GpeVinculom
     */
    private function buscaVinculom($objVinculoh)
    {
        $params = array('P_NRVINCULOM' => $objVinculoh->getNrvinculom(),
            'P_NRORG' => $objVinculoh->getNrorg());
        $where = "NRVINCULOM = :P_NRVINCULOM AND NRORG = :P_NRORG";
        return GpeVinculom::getObjectByWhere($where, $params);
    }

    /**Busca Situação funcional
     * @param integer $nrsitufuncm
     * @return GpeSitufunch|bool
     */
    private function buscaSitufuncm($nrsitufuncm)
    {
        $params = array('P_NRSITUFUNCM' => $nrsitufuncm);
        $where = "NRSITUFUNCH IN ( SELECT NRSITUFUNCH FROM GPE_SITUFUNCH WHERE NRSITUFUNCM = :P_NRSITUFUNCM AND DTEFETIVA = (SELECT MAX(MAX_SITUFUNCH.DTEFETIVA) FROM GPE_SITUFUNCH MAX_SITUFUNCH WHERE MAX_SITUFUNCH.NRSITUFUNCM = GPE_SITUFUNCH.NRSITUFUNCM AND MAX_SITUFUNCH.NRSITUFUNCM = :P_NRSITUFUNCM))";
        return GpeSitufunch::getObjectByWhere($where, $params);
    }

    /**Busca o vínculo
     * @param String $where
     * @return GpeVinculoh
     */
    private function buscaVinculoh($where)
    {
        $params = array('P_NRORG' => $this->nrorg,
            'P_DTMESCOMPETENC' => $this->dtmescompetencia);

        $where = "$where AND NRORG = :P_NRORG AND NRVINCULOH IN (SELECT NRVINCULOH FROM GPE_VINCULOH INNER JOIN GPE_VINCULOM ON GPE_VINCULOM.NRORG = GPE_VINCULOH.NRORG AND GPE_VINCULOM.NRVINCULOM = GPE_VINCULOH.NRVINCULOM WHERE TRUNC(GPE_VINCULOM.DTADMISSAOVINC, 'DD') <= :P_DTMESCOMPETENC AND (TRUNC(GPE_VINCULOM.DTRESCISAOVINC, 'DD') >= :P_DTMESCOMPETENC OR GPE_VINCULOM.DTRESCISAOVINC IS NULL) AND DTMESCOMPETENC = (SELECT MAX(MAX_VINCULOH.DTMESCOMPETENC) FROM GPE_VINCULOH MAX_VINCULOH WHERE MAX_VINCULOH.NRORG = GPE_VINCULOH.NRORG AND MAX_VINCULOH.NRVINCULOM = GPE_VINCULOH.NRVINCULOM AND TRUNC(MAX_VINCULOH.DTMESCOMPETENC, 'DD') <= :P_DTMESCOMPETENC AND MAX_VINCULOH.NRORG = :P_NRORG))";
        return GpeVinculoh::getObjectsByWhere($where, $params, 'NRESTRUTGEREN, NRVINCULOM');
    }

    /**Busca férias
     * @param $objVinculom GpeVinculom
     * @return bool
     */
    private function buscaFerias($objVinculom)
    {
        $params = array('P_NRORG'               => $this->nrorg,
                        'P_NRVINCULOM'          => $objVinculom->getNrvinculom(),
                        'P_DTMESCOMPETENCIA'    => $this->dtmescompetencia);
        $where = "NRORG = :P_NRORG AND NRGOZOFERIAS = (SELECT NRGOZOFERIAS FROM FPA_GOZOFERIAS WHERE NRORG = :P_NRORG AND DTINIGOZOFERIAS = (SELECT MAX(MAX_GOZOFERIAS.DTINIGOZOFERIAS) FROM FPA_GOZOFERIAS MAX_GOZOFERIAS WHERE MAX_GOZOFERIAS.NRORG = FPA_GOZOFERIAS.NRORG AND MAX_GOZOFERIAS.NRFERIAS = FPA_GOZOFERIAS.NRFERIAS AND(MAX_GOZOFERIAS.DTINIGOZOFERIAS BETWEEN TRUNC(TO_DATE(:P_DTMESCOMPETENCIA),'MONTH') AND LAST_DAY(TO_DATE(:P_DTMESCOMPETENCIA)) OR MAX_GOZOFERIAS.DTFIMGOZOFERIAS BETWEEN TRUNC(TO_DATE(:P_DTMESCOMPETENCIA),'MONTH') AND LAST_DAY(TO_DATE(:P_DTMESCOMPETENCIA))) AND NRFERIAS IN (SELECT NRFERIAS FROM FPA_FERIAS WHERE NRORG = :P_NRORG AND NRVINCULOM = :P_NRVINCULOM )))";
        $result = FpaGozoferias::getObjectsByWhere($where, $params);
        if($result){
            return true;
        }else{
            return false;
        }
    }

    /**Busca alterações de situações funcionais
     *
     * @param $objVinculom GpeVinculom
     * @return int
     */
    public function buscaAlteSitufunc($objVinculom)
    {
        $params = array('P_NRORG'               => $this->nrorg,
                        'P_NRVINCULOM'          => $objVinculom->getNrvinculom(),
                        'P_DTMESCOMPETENCIA'    => $this->dtmescompetencia);
        $where = "NRORG = :P_NRORG AND NRVINCULOM = :P_NRVINCULOM AND NRALTESITUFUNC = (SELECT MAX(NRALTESITUFUNC) FROM GPE_ALTESITUFUNC WHERE GPE_ALTESITUFUNC.NRORG = :P_NRORG AND NRVINCULOM = :P_NRVINCULOM AND DTINISITUFUNC = (SELECT MAX(MAX_ALTESITUFUNC.DTINISITUFUNC) FROM GPE_ALTESITUFUNC MAX_ALTESITUFUNC WHERE MAX_ALTESITUFUNC.DTINISITUFUNC <= :P_DTMESCOMPETENCIA AND MAX_ALTESITUFUNC.NRORG = GPE_ALTESITUFUNC.NRORG AND MAX_ALTESITUFUNC.NRVINCULOM = GPE_ALTESITUFUNC.NRVINCULOM ))";
        $result = GpeAltesitufunc::getObjectByWhere($where, $params);

        if($result){
            return $result->getNrsitufuncm();
        }else{
            return 1;
        }
    }

    private static function _removeCaracteresEspeciais($string){
        $string = preg_replace(self::APENAS_CARACTERES_ALFANUMERICOS, self::CARACTER_VAZIO, $string);
        $string = str_replace('  ', ' ',$string);

        return $string;
    }


} 